
package eu.dnetlib.dhp.oa.graph.dump.eosc;

import static eu.dnetlib.dhp.common.SparkSessionSupport.runWithSparkSession;

import java.io.Serializable;
import java.util.Optional;

import org.apache.commons.io.IOUtils;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.function.FilterFunction;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.SparkSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.dnetlib.dhp.application.ArgumentApplicationParser;
import eu.dnetlib.dhp.oa.graph.dump.Utils;
import eu.dnetlib.dhp.oa.model.graph.GraphResult;
import eu.dnetlib.dhp.schema.oaf.Relation;

/**
 * @author miriam.baglioni
 * @Date 27/07/22
 */
public class SelectEoscRelationsStep2 implements Serializable {
	private static final Logger log = LoggerFactory.getLogger(SelectEoscRelationsStep2.class);

	public static void main(String[] args) throws Exception {
		String jsonConfiguration = IOUtils
			.toString(
				SelectEoscRelationsStep2.class
					.getResourceAsStream(
						"/eu/dnetlib/dhp/oa/graph/dump/reletion_selection_parameters.json"));

		final ArgumentApplicationParser parser = new ArgumentApplicationParser(jsonConfiguration);
		parser.parseArgument(args);

		Boolean isSparkSessionManaged = Optional
			.ofNullable(parser.get("isSparkSessionManaged"))
			.map(Boolean::valueOf)
			.orElse(Boolean.TRUE);
		log.info("isSparkSessionManaged: {}", isSparkSessionManaged);

		final String inputPath = parser.get("sourcePath");
		log.info("inputPath: {}", inputPath);

		final String resultPath = parser.get("resultPath");
		log.info("resultPath: {}", resultPath);

		SparkConf conf = new SparkConf();

		runWithSparkSession(
			conf,
			isSparkSessionManaged,
			spark -> {
				Utils.removeOutputDir(spark, resultPath + "/relation");
				selectRelations(spark, inputPath, resultPath + "/relation", resultPath);
			});
	}

	private static void selectRelations(SparkSession spark, String inputPath, String outputPath, String resultPath) {
		Dataset<GraphResult> results = Utils
			.readPath(spark, resultPath + "/publication", GraphResult.class)
			.union(
				Utils
					.readPath(spark, resultPath + "/dataset", GraphResult.class))
			.union(
				Utils
					.readPath(spark, resultPath + "/software", GraphResult.class))
			.union(
				Utils
					.readPath(spark, resultPath + "/otherresearchproduct", GraphResult.class));

		Dataset<Relation> relations = Utils
			.readPath(spark, inputPath + "/relation", Relation.class)
			.filter(
				(FilterFunction<Relation>) r -> !r.getDataInfo().getDeletedbyinference() &&
					!r.getDataInfo().getInvisible());

	}

}
